#include "pch.h"

CTidget* CTidget::CreateObject(wstring cls)
{
	if (cls == L"LCDTest") return new CLCDTest();
	if (cls == L"TextClip") return new CTextClip();
	if (cls == L"ColorTable") return new CColorTable(); 
	if (cls == L"MoveCursor") return new CMoveCursor();
	if (cls == L"SizeMeter") return new CSizeMeter();
	if (cls == L"Tetris") return new CTetris();
	if (cls == L"OneImage") return new COneImage();
	if (cls == L"EnumFont") return new CEnumFont();
	if (cls == L"Spy") return new CSpy();

	return nullptr;
}

CTidget* CTidget::CreateDefaultObject(wstring cls)
{
	CTidget* pTidget = CreateObject(cls);

	int ti;
	for (ti = 0;; ti++) {
		if (arTidgetInfo[ti].cls == L"") return nullptr;
		if (arTidgetInfo[ti].cls == cls) break;
	}

	pTidget->dget_cls = cls;
	if (arTidgetInfo[ti].allowMulti == false) {
		pTidget->dget_id = cls;
	} else {
		bool bFind;
		wstring id;
		for (int num = 1;; num++) {
			if (num == 1) {
				id = cls;
			} else {
				TCHAR tnum[10];
				wsprintf(tnum, L"%d", num);
				id = cls + L"_" + tnum;
			}
			bFind = false;
			for (size_t ti = 0; ti < arTidget.size(); ti++) {
				if (arTidget[ti]->dget_id == id) {
					bFind = true;
					break;
				}
			}
			if (bFind == false) break;
		}

		pTidget->dget_id = id;
	}

	pTidget->dget_visible = true;

	return pTidget;
}

void CTidget::Serialize(TCHAR* pack)
{
	wsprintf(pack, L"%s%s%d",
		dget_cls.c_str(), dget_id.c_str(), dget_visible);
}

CTidget* CTidget::Deserialize(LPCTSTR pack)
{
	TCHAR token[MAX_PATH];
	CWidget::GetPackToken(pack, 0, token);
	CTidget* pTidget = CreateObject(token);
	pTidget->dget_cls = token;
	CWidget::GetPackToken(pack, 1, token); pTidget->dget_id = token;
	CWidget::GetPackToken(pack, 2, token); pTidget->dget_visible = _ttoi(token);

	return pTidget;
}


LRESULT CTidget::OnLButtonDown(WPARAM wParam, LPARAM lParam) {
	SetCapture(win_hWnd);
	return 0;
}

LRESULT CTidget::OnLButtonUp(WPARAM wParam, LPARAM lParam) {
	if (GetCapture() != win_hWnd) return 0;
	ReleaseCapture();

	POINT pt = { (int)(short)LOWORD(lParam) , (int)(short)HIWORD(lParam) };
	RECT crt;
	GetClientRect(win_hWnd, &crt);
	if (PtInRect(&crt, pt) == false) return 0;

	OnClick();
	return 0;
}

void CLCDTestWin::Create(HWND hParent) {
	// ׻ .  μ this 
	CreateWindowEx(WS_EX_TOPMOST, L"CWindow", L"", WS_POPUP | WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		hParent, (HMENU)NULL, g_hInst, this);
}

LRESULT CLCDTestWin::OnMessage(UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch (iMessage)
	{
	case WM_LBUTTONDOWN:
		RotateColor();
		return 0;
	case WM_RBUTTONDOWN:
		RotateMonitor();
		return 0;
	}

	return CWindow::OnMessage(iMessage, wParam, lParam);
}

LRESULT CLCDTestWin::OnCreate(WPARAM wParam, LPARAM lParam)
{
	arColor[0] = RGB(0, 0, 0);
	arColor[1] = RGB(255, 255, 255);
	arColor[2] = RGB(255, 0, 0);
	arColor[3] = RGB(0, 255, 0);
	arColor[4] = RGB(0, 0, 255);
	arColor[5] = RGB(255, 255, 0);
	arColor[6] = RGB(0, 255, 255);
	arColor[7] = RGB(255, 0, 255);
	colorIdx = 0;
	displayText = true;
	animate = false;

	//  찡 ִ ͸  ä. 
	nowMonitor = GetMonitorOfWindow(mainWindow.win_hWnd);
	RECT rt = arMonitor[nowMonitor].rcMonitor;
	SetWindowPos(win_hWnd, NULL, rt.left, rt.top, rt.right - rt.left, rt.bottom - rt.top,
		SWP_NOZORDER);
	return 0;
}

LRESULT CLCDTestWin::OnPaint(WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	HBRUSH hBrush;
	RECT crt, art;
	static LPCTSTR Mes = TEXT("LCD ׽Ʈ\r\n Ŭ, ̽: \r\n Ŭ, Tab: ̵\r\n")
		TEXT("Esc:, Enter:ȳ ڿ \r\nRGBWKYCM:  \r\nA:ִϸ̼");

	GetClientRect(win_hWnd, &crt);
	hdc = BeginPaint(win_hWnd, &ps);
	if (animate) {
		for (int x = -aniOffset, c = aniIdx; x < crt.right;) {
			hBrush = CreateSolidBrush(arColor[c]);
			SetRect(&art, x, crt.top, x + tileWidth, crt.bottom);
			FillRect(hdc, &art, hBrush);
			DeleteObject(hBrush);
			x += tileWidth;
			moveNext(c, Count(arColor));
		}
	} else {
		hBrush = CreateSolidBrush(arColor[colorIdx]);
		FillRect(hdc, &crt, hBrush);
		DeleteObject(hBrush);

		if (displayText) {
			if (colorIdx == 0) {
				SetTextColor(hdc, RGB(255, 255, 255));
			}
			SetBkMode(hdc, TRANSPARENT);
			crt.top = crt.bottom / 2;
			DrawText(hdc, Mes, -1, &crt, DT_CENTER);
		}
	}

	EndPaint(win_hWnd, &ps);

	return 0;
}

LRESULT CLCDTestWin::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	switch (wParam) {
	case VK_SPACE:
		RotateColor();
		break;
	case VK_TAB:
		RotateMonitor();
		break;
	case VK_ESCAPE:
		//  ı  üʿ 
		if (animate) {
			KillTimer(win_hWnd, 1);
		}
		DestroyWindow(win_hWnd);
		Dispose();
		break;
	case VK_RETURN:
		displayText = !displayText;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'R':
		colorIdx = 2;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'G':
		colorIdx = 3;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'B':
		colorIdx = 4;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'W':
		colorIdx = 1;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'K':
		colorIdx = 0;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'Y':
		colorIdx = 5;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'C':
		colorIdx = 6;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'M':
		colorIdx = 7;
		InvalidateRect(win_hWnd, NULL, FALSE);
		break;
	case 'A':
		animate = !animate;
		if (animate) {
			SetTimer(win_hWnd, 1, 10, NULL);
			aniIdx = 0;
			aniOffset = 0;
			tileWidth = 100;
		} else {
			KillTimer(win_hWnd, 1);
			InvalidateRect(win_hWnd, NULL, FALSE);
		}
		break;
	}

	return 0;
}

LRESULT CLCDTestWin::OnTimer(WPARAM wParam, LPARAM lParam)
{
	if (aniOffset == tileWidth - 1) {
		aniOffset = 0;
		moveNext(aniIdx, Count(arColor));
	} else {
		aniOffset++;
	}
	InvalidateRect(win_hWnd, NULL, FALSE);
	return 0;
}

LRESULT CLCDTestWin::RotateColor()
{
	moveNext(colorIdx, Count(arColor));
	InvalidateRect(win_hWnd, NULL, FALSE);

	return 0;
}

LRESULT CLCDTestWin::RotateMonitor()
{
	if (monitorCount > 1) {
		moveNext(nowMonitor, monitorCount);
		RECT rt = arMonitor[nowMonitor].rcMonitor;
		SetWindowPos(win_hWnd, NULL, rt.left, rt.top, rt.right - rt.left, rt.bottom - rt.top,
			SWP_NOZORDER);
	}

	return 0;
}

LRESULT CLCDTest::OnCreate(WPARAM wParam, LPARAM lParam) {
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_LCDTEST));
	return 0;
}

LRESULT CLCDTest::OnDestroy(WPARAM wParam, LPARAM lParam) {
	DeleteObject(icon);
	return 0;
}

void CLCDTest::OnDraw(HDC hdc, int width, int height) {
	int wh = trayWindow.tidgetSize;
	StretchBitmap(hdc, 0, 0, wh, wh, icon);
}

void CLCDTest::OnClick() {
	CLCDTestWin* pWin = new CLCDTestWin();
	pWin->delegateDispose = true;
	pWin->Create(NULL);
}

LRESULT CTextClip::OnMessage(UINT iMessage, WPARAM wParam, LPARAM lParam) {
	switch (iMessage)
	{
	case WM_HOTKEY:
		ClipboardTextOnly();
		return 0;
	}

	return CWindow::OnMessage(iMessage, wParam, lParam);
}

LRESULT CTextClip::OnCreate(WPARAM wParam, LPARAM lParam)
{
	bitOn = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_CLIPTEXTON));
	bitOff = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_CLIPTEXTOFF));

	clipToText = setting.get(dget_id, L"clipToText", 1);
	clipCount = 0;

	if (clipToText) {
		RegisterHotKey(win_hWnd, 0, MOD_SHIFT | MOD_CONTROL, 'C');
	}

	return 0;
}

LRESULT CTextClip::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	DeleteObject(bitOn);
	DeleteObject(bitOff);
	UnregisterHotKey(win_hWnd, 0);
	return 0;
}

void CTextClip::OnDraw(HDC hdc, int width, int height)
{
	int wh = trayWindow.tidgetSize;
	if (clipToText) {
		StretchBitmap(hdc, 0, 0, wh, wh, bitOn);
	} else {
		StretchBitmap(hdc, 0, 0, wh, wh, bitOff);
	}
}

void CTextClip::OnClick()
{
	clipToText = !clipToText;
	if (clipToText) {
		RegisterHotKey(win_hWnd, 0, MOD_SHIFT | MOD_CONTROL, 'C');
	} else {
		UnregisterHotKey(win_hWnd, 0);
	}
	InvalidateRect(win_hWnd, NULL, FALSE);
	setting.set(dget_id, L"clipToText", clipToText);
}

// ؽƮ 
void CTextClip::ClipboardTextOnly()
{
	HGLOBAL hmem;
	UINT Format = 0;
	BOOL bOther = FALSE;
	wchar_t* p, * p2;

	// ϴ ؽƮ  ־ Ѵ.
	if (!IsClipboardFormatAvailable(CF_UNICODETEXT)) {
		return;
	}

	// ؽƮܿ ٸ ˵  ־ Ѵ.
	OpenClipboard(mainWindow.win_hWnd);
	for (;;) {
		Format = EnumClipboardFormats(Format);
		if (Format == 0) break;
		if (Format != CF_TEXT && Format != CF_OEMTEXT &&
			Format != CF_UNICODETEXT && Format != CF_LOCALE) {
			bOther = TRUE;
			break;
		}
	}
	CloseClipboard();

	// ؽƮ ִٸ ȯ ʿ䰡 .
	if (bOther == FALSE) {
		return;
	}

	OpenClipboard(mainWindow.win_hWnd);
	hmem = GetClipboardData(CF_UNICODETEXT);
	p = (wchar_t*)GlobalLock(hmem);
	p2 = (wchar_t*)calloc(lstrlenW(p) + 1, sizeof(wchar_t));
	lstrcpy(p2, p);
	GlobalUnlock(hmem);
	CloseClipboard();

	CopyClipboardText(p2);
	free(p2);
	clipCount++;
}

LRESULT CColorTable::OnCreate(WPARAM wParam, LPARAM lParam) {
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_COLORTABLE));
	return 0;
}

LRESULT CColorTable::OnDestroy(WPARAM wParam, LPARAM lParam) {
	DeleteObject(icon);
	return 0;
}

void CColorTable::OnDraw(HDC hdc, int width, int height) {
	int wh = trayWindow.tidgetSize;
	StretchBitmap(hdc, 0, 0, wh, wh, icon);
}

void CColorTable::OnClick() {
	CColorTableWin* pWin = new CColorTableWin();
	pWin->delegateDispose = true;
	pWin->Create(NULL);
}

// ǥ â. 
// todo : Űε ¿ ̵ϴ  ߰ 
void CColorTableWin::Create(HWND hParent)
{
	// ּȭ   
	CreateWindow(TEXT("CWindow"), L"ǥ", WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU |
		WS_THICKFRAME | WS_MAXIMIZEBOX | WS_VISIBLE,
		0, 0, 0, 0, hParent, (HMENU)NULL, g_hInst, this);

	//   ġ .
	WINDOWPLACEMENT wndpl;
	wndpl.length = sizeof(WINDOWPLACEMENT);
	wndpl.flags = 0;
	wndpl.rcNormalPosition.left = setting.get(L"ColorTableApp", L"left", 10);
	wndpl.rcNormalPosition.top = setting.get(L"ColorTableApp", L"top", 10);
	wndpl.rcNormalPosition.right = setting.get(L"ColorTableApp", L"right", 1500);
	wndpl.rcNormalPosition.bottom = setting.get(L"ColorTableApp", L"bottom", 1024);
	wndpl.showCmd = setting.get(L"ColorTableApp", L"showCmd", SW_SHOWNORMAL);
	SetWindowPlacement(win_hWnd, &wndpl);
}

LRESULT CColorTableWin::OnCreate(WPARAM wParam, LPARAM lParam)
{
	hList = CreateWindow(TEXT("listbox"), NULL, WS_CHILD | WS_VISIBLE | WS_BORDER |
		LBS_NOTIFY, 5, 5, 130, 150, win_hWnd, (HMENU)CT_LIST, g_hInst, NULL);
	SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)L" ǥػ()");
	SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)L" ǥػ(̸)");
	SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)L" ");
	SendMessage(hList, LB_SETCURSEL, 0, 0);
	SendMessage(hList, WM_SETFONT, (WPARAM)mainFont[1], MAKELPARAM(FALSE, 0));

	// 4  ư ġ
	for (int i = 0; i < 4; i++) {
		hBtnCopy[i] = CreateWindow(TEXT("button"), TEXT(""), WS_CHILD | WS_VISIBLE |
			BS_PUSHBUTTON, 5, 480 + i * 30, 130, 25, win_hWnd, (HMENU)(CT_COPY1 + i), g_hInst, NULL);
		SendMessage(hBtnCopy[i], WM_SETFONT, (WPARAM)mainFont[1], MAKELPARAM(FALSE, 0));
	}

	PrepareColor(0);
	selIdx = 0;
	ChangeSelColor(0);
	return 0;
}

LRESULT CColorTableWin::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	//   ġ 
	WINDOWPLACEMENT wndpl;
	wndpl.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(win_hWnd, &wndpl);

	setting.set(L"ColorTableApp", L"left", (int)wndpl.rcNormalPosition.left);
	setting.set(L"ColorTableApp", L"top", (int)wndpl.rcNormalPosition.top);
	setting.set(L"ColorTableApp", L"right", (int)wndpl.rcNormalPosition.right);
	setting.set(L"ColorTableApp", L"bottom", (int)wndpl.rcNormalPosition.bottom);
	setting.set(L"ColorTableApp", L"showCmd", (int)wndpl.showCmd);

	return 0;
}

LRESULT CColorTableWin::OnSize(WPARAM wParam, LPARAM lParam)
{
	CWindow::OnSize(wParam, lParam);

	CalcTileSize();
	return 0;
}

LRESULT CColorTableWin::OnCommand(WPARAM wParam, LPARAM lParam)
{
	TCHAR str[128];

	switch (LOWORD(wParam)) {
	case CT_LIST:
		switch (HIWORD(wParam)) {
		case LBN_SELCHANGE:
			PrepareColor(SendMessage(hList, LB_GETCURSEL, 0, 0));
			CalcTileSize();
			InvalidateRect(win_hWnd, NULL, TRUE);
			return 0;
		}
		break;
	case CT_COPY1:
	case CT_COPY2:
	case CT_COPY3:
	case CT_COPY4:
		GetWindowText((HWND)lParam, str, 128);
		CopyClipboardText(str);
		break;
	}

	return 0;
}

void CColorTableWin::OnDraw(HDC hdc, int width, int height)
{
	int x = leftMargin;
	int y = 0;
	HBRUSH brush, oldBrush;
	TCHAR value[128];
	HFONT oldFont;
	int rgb;

	//     
	rgb = RGB(GetBValue(arColor[selIdx].color), GetGValue(arColor[selIdx].color),
		GetRValue(arColor[selIdx].color));
	brush = CreateSolidBrush(rgb);
	oldBrush = (HBRUSH)SelectObject(hdc, brush);
	Rectangle(hdc, 5, 170, 135, 300);
	DeleteObject(SelectObject(hdc, oldBrush));

	//  Ʒ   
	rgb = RGB(GetBValue(arColor[prevSel].color), GetGValue(arColor[prevSel].color),
		GetRValue(arColor[prevSel].color));
	brush = CreateSolidBrush(rgb);
	oldBrush = (HBRUSH)SelectObject(hdc, brush);
	Rectangle(hdc, 5, 305, 135, 435);
	DeleteObject(SelectObject(hdc, oldBrush));

	SetBkMode(hdc, TRANSPARENT);
	oldFont = (HFONT)SelectObject(hdc, mainFont[1]);
	TextOutSz(hdc, 5, 455, L" ");

	SetTextAlign(hdc, TA_CENTER);
	if (tile_width <= 80) {
		oldFont = (HFONT)SelectObject(hdc, mainFont[0]);
	}

	for (int idx = 0; ; idx++) {
		if (lstrlen(arColor[idx].name) == 0) break;

		//     
		if (idx == selIdx) {
			brush = CreateSolidBrush(RGB(0, 0, 255));
			oldBrush = (HBRUSH)SelectObject(hdc, brush);
			Rectangle(hdc, x, y, x + tile_width, y + tile_height);
			DeleteObject(SelectObject(hdc, oldBrush));
			SetTextColor(hdc, RGB(255, 255, 255));
		} else {
			SetTextColor(hdc, RGB(0, 0, 0));
		}

		// 16  BGR̾  ٲ㼭  . 
		rgb = RGB(GetBValue(arColor[idx].color), GetGValue(arColor[idx].color),
			GetRValue(arColor[idx].color));
		brush = CreateSolidBrush(rgb);
		oldBrush = (HBRUSH)SelectObject(hdc, brush);

		if (namedColor) {
			TextOutSz(hdc, x + tile_width / 2, y + 2, arColor[idx].name);
			// ¿ 4ȼ е 
			Rectangle(hdc, x + 4, y + 20, x + tile_width - 4, y + tile_height - 20);
		} else {
			Rectangle(hdc, x + 4, y + 4, x + tile_width - 4, y + tile_height - 20);
		}
		DeleteObject(SelectObject(hdc, oldBrush));
		wsprintf(value, L"%06X", arColor[idx].color);
		TextOutSz(hdc, x + tile_width / 2, y + tile_height - 20, value);

		x += tile_width;
		if (x + tile_width > width) {
			x = leftMargin;
			y += tile_height;
		}
	}

	SelectObject(hdc, oldFont);
}

LRESULT CColorTableWin::OnLButtonDown(WPARAM wParam, LPARAM lParam)
{
	POINT pt;
	int idx;

	pt.x = (int)(short)LOWORD(lParam);
	pt.y = (int)(short)HIWORD(lParam);

	//   Ŭص ƹ ó  ʴ´.
	if (pt.x <= leftMargin) return 0;

	// ǥκ Ŭ Ÿ 
	idx = (pt.y / tile_height) * perLine + (pt.x - leftMargin) / tile_width;

	// ȭ Ʒ Ŭ   
	if (idx >= colorCount) idx = colorCount - 1;

	ChangeSelColor(idx);
	return 0;
}

void CColorTableWin::PrepareColor(int colorType)
{
	int i;

	switch (colorType)
	{
	case 0:
	case 1:
		for (i = 0; ; i++) {
			if (lstrlen(arStandardColor[i].name) == 0) break;
			lstrcpy(arColor[i].name, arStandardColor[i].name);
			arColor[i].color = arStandardColor[i].color;
		}
		lstrcpy(arColor[i].name, L"");
		colorCount = i;
		if (colorType == 1) {
			qsort(arColor, i, sizeof(sOneColor), colorNameCompare);
		}
		namedColor = true;
		selIdx = prevSel = 0;
		break;
	case 2:
		i = 0;
		for (int r = 0; r <= 0xff; r += 0x33) {
			for (int g = 0; g <= 0xff; g += 0x33) {
				for (int b = 0; b <= 0xff; b += 0x33) {
					wsprintf(arColor[i].name, L"%02X%02X%02X", r, g, b);
					arColor[i].color = r * 0x10000 + g * 0x100 + b;
					i++;
				}
			}
		}
		lstrcpy(arColor[i].name, L"");
		colorCount = i;
		namedColor = false;
		selIdx = prevSel = 0;

		break;
	}
}

int CColorTableWin::colorNameCompare(const void* a, const void* b)
{
	CColorTableWin::sOneColor ra, rb;
	ra = *(CColorTableWin::sOneColor*)a;
	rb = *(CColorTableWin::sOneColor*)b;

	if (lstrcmp(ra.name, rb.name) > 0) {
		return 1;
	}

	return -1;
}

void CColorTableWin::CalcTileSize()
{
	RECT crt;
	GetClientRect(win_hWnd, &crt);
	int width = crt.right - leftMargin;
	int height = crt.bottom;

	// ٴ 16. ּ ȼ   ٴ  .  1280 ϸ 16 ۾.
	tile_width = width / 16;
	if (tile_width < 80) tile_width = 80;
	perLine = width / tile_width;

	// 0  
	if (perLine == 0) perLine = 1;

	// ʿ   ̸  ϵ ּ  .
	int needRow = colorCount / perLine + 1;
	tile_height = height / needRow;
	if (tile_height < 50) tile_height = 50;
}

void CColorTableWin::ChangeSelColor(int idx)
{
	//  ϱ   
	prevSel = selIdx;
	selIdx = idx;

	int color = arColor[selIdx].color;
	TCHAR text[128];

	SetWindowText(hBtnCopy[0], arColor[selIdx].name);
	wsprintf(text, L"RGB(%d, %d, %d)", GetBValue(color), GetGValue(color),
		GetRValue(color));
	SetWindowText(hBtnCopy[1], text);
	wsprintf(text, L"0X%0X", color);
	SetWindowText(hBtnCopy[2], text);
	wsprintf(text, L"#%0X", color);
	SetWindowText(hBtnCopy[3], text);

	InvalidateRect(win_hWnd, NULL, TRUE);
}

LRESULT CMoveCursor::OnCreate(WPARAM wParam, LPARAM lParam) {
	bitOn = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_MOVECURSORON));
	bitOff = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_MOVECURSOROFF));

	LoadSetting();
	RejectCount = 0;
	if (bCursorMove) {
		SetTimer(win_hWnd, 1, TimerInterval, NULL);
	}
	return 0;
}

LRESULT CMoveCursor::OnDestroy(WPARAM wParam, LPARAM lParam) {
	setting.set(dget_id, L"moveCursor", bCursorMove);
	DeleteObject(bitOn);
	DeleteObject(bitOff);
	KillTimer(win_hWnd, 1);
	return 0;
}

void CMoveCursor::OnDraw(HDC hdc, int width, int height)
{
	int wh = trayWindow.tidgetSize;
	if (bCursorMove) {
		StretchBitmap(hdc, 0, 0, wh, wh, bitOn);
	} else {
		StretchBitmap(hdc, 0, 0, wh, wh, bitOff);
	}
}

void CMoveCursor::OnClick()
{
	bCursorMove = !bCursorMove;
	if (bCursorMove) {
		KillTimer(win_hWnd, 1);
	} else {
		SetTimer(win_hWnd, 1, TimerInterval, NULL);
	}
	SaveSetting();
	InvalidateRect(win_hWnd, NULL, FALSE);
}

LRESULT CMoveCursor::OnTimer(WPARAM wParam, LPARAM lParam)
{
	POINT pt;

	GetCursorPos(&pt);
	//  ڸ ִ  ð  
	if (pt.x == OldCursorPt.x && pt.y == OldCursorPt.y) {
		CursorKeepSec += TimerInterval / 1000;
		// ũ ̹  ð Ǹ  Ŀ ĭ  ű 
		if (CursorKeepSec >= CursorInterval) {
			CursorKeepSec = 0;
			mouse_event(MOUSEEVENTF_MOVE, 1, 0, 0, 0);
			RejectCount++;
		}
		// ġ ٲ  ð  
	} else {
		CursorKeepSec = 0;
		OldCursorPt = pt;
	}
	return 0;
}

LRESULT CMoveCursor::OnContextMenu(WPARAM wParam, LPARAM lParam)
{
	if (!IsWindow(hOptionPopup)) {
		hOptionPopup = CreateDialogParam(g_hInst, MAKEINTRESOURCE(IDD_MOVECURSOR),
			win_hWnd, MoveCursorDlgProc, (LPARAM)this);
		ShowWindow(hOptionPopup, SW_SHOW);
	}
	return 0;
}

INT_PTR CALLBACK CMoveCursor::MoveCursorDlgProc(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	static CMoveCursor* pTidget;
	TCHAR str[128];
	RECT wrt;

	switch (iMessage) {
	case WM_INITDIALOG:
		pTidget = (CMoveCursor*)lParam;
		GetWindowRect(trayWindow.win_hWnd, &wrt);
		MovePopupToSide(hDlg, wrt.left, wrt.top);

		//  ݴ   , Ʒ ̴. 
		SendDlgItemMessage(hDlg, IDC_SPINMOVECURSORCYCLE, UDM_SETRANGE, 0, MAKELPARAM(600, 60));

		wsprintf(str, L"̵ Ƚ : %d", pTidget->RejectCount);
		SetDlgItemText(hDlg, IDC_STMOVECURSORCOUNT, str);
		if (pTidget->bCursorMove) {
			CheckDlgButton(hDlg, IDC_CHKMOVECURSOR, BST_CHECKED);
		}
		SetDlgItemInt(hDlg, IDC_EDMOVECURSORCYCLE, pTidget->CursorInterval, FALSE);
		return TRUE;
	case WM_NCACTIVATE:
		if (wParam == FALSE && hOptionPopup != NULL) {
			pTidget->CursorInterval = GetDlgItemInt(hDlg, IDC_EDMOVECURSORCYCLE, NULL, FALSE);
			pTidget->TimerInterval = pTidget->CursorInterval * 100;
			if (IsDlgButtonChecked(hDlg, IDC_CHKMOVECURSOR) == BST_CHECKED) {
				pTidget->bCursorMove = true;
				SetTimer(pTidget->win_hWnd, 1, pTidget->TimerInterval, NULL);
			} else {
				pTidget->bCursorMove = false;
				KillTimer(pTidget->win_hWnd, 1);
			}

			pTidget->SaveSetting();

			hOptionPopup = NULL;
			DestroyWindow(hDlg);
		}
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDCANCEL:
			hOptionPopup = NULL;
			DestroyWindow(hDlg);
			return TRUE;
		}
		break;
	}
	return FALSE;
}

void CMoveCursor::LoadSetting()
{
	bCursorMove = setting.get(dget_id, L"moveCursor", 0);
	CursorInterval = setting.get(dget_id, L"CursorInterval", 180);
	// Ÿ̸Ӵ ̵ֱ 1/10 ڵ Ѵ. 180ʸ 18ʿ ѹ Ȯϸ ϴ.
	//  ɼ µ ڰ ̷    ʿ . 
	TimerInterval = CursorInterval * 100;
	CursorKeepSec = 0;
}

void CMoveCursor::SaveSetting()
{
	setting.set(dget_id, L"bCursorMove", bCursorMove);
	setting.set(dget_id, L"CursorInterval", CursorInterval);
}

LRESULT CSizeMeter::OnCreate(WPARAM wParam, LPARAM lParam) {
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_SIZEMETER));
	return 0;
}

LRESULT CSizeMeter::OnDestroy(WPARAM wParam, LPARAM lParam) {
	DeleteObject(icon);
	return 0;
}

void CSizeMeter::OnDraw(HDC hdc, int width, int height) {
	int wh = trayWindow.tidgetSize;
	StretchBitmap(hdc, 0, 0, wh, wh, icon);
}

void CSizeMeter::OnClick() {
	CSizeMeterWin* pWin = new CSizeMeterWin();
	pWin->delegateDispose = true;
	pWin->Create(NULL);
}

void CSizeMeterWin::Create(HWND hParent)
{
	CreateWindow(L"CWindow", L"SizeMeter", WS_OVERLAPPEDWINDOW | WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, (HMENU)NULL, g_hInst, this);
}

LRESULT CSizeMeterWin::OnSize(WPARAM wParam, LPARAM lParam)
{
	CWindow::OnSize(wParam, lParam);

	InvalidateRect(win_hWnd, NULL, TRUE);
	return 0;
}

void CSizeMeterWin::OnDraw(HDC hdc, int width, int height)
{
	TCHAR str[111];

	SetBkMode(hdc, TRANSPARENT);
	SetTextAlign(hdc, TA_CENTER);
	wsprintf(str, TEXT("%d x %d"), width, height);
	TextOutSz(hdc, width / 2, height / 2, str);
	lstrcpy(str, TEXT("1 ~ 9 : ǥ ũ"));
	TextOutSz(hdc, width / 2, height / 2 + 30, str);
}

LRESULT CSizeMeterWin::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	RECT crt;
	int w, h;

	w = 0;
	switch (wParam) {
	case '1':
		w = 320; h = 240; break;
	case '2':
		w = 640; h = 480; break;
	case '3':
		w = 800; h = 600; break;
	case '4':
		w = 1024; h = 768; break;
	case '5':
		w = 1280; h = 1024; break;
	case '6':
		w = 1600; h = 1200; break;
	case '7':
		w = 1920; h = 1080; break;
	case '8':
		w = 2560; h = 1440; break;
	case '9':
		w = 3840; h = 2160; break;
	}
	if (w != 0)
	{
		SetRect(&crt, 0, 0, w, h);
		DWORD Style = GetWindowLong(win_hWnd, GWL_STYLE);
		DWORD ExStyle = GetWindowLong(win_hWnd, GWL_EXSTYLE);

		AdjustWindowRectEx(&crt, Style, GetMenu(win_hWnd) != NULL, ExStyle);
		SetWindowPos(win_hWnd, NULL, 0, 0, crt.right - crt.left, crt.bottom - crt.top,
			SWP_NOMOVE | SWP_NOZORDER);
	}
	return 0;
}

LRESULT CTetris::OnCreate(WPARAM wParam, LPARAM lParam)
{
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_TETRIS));
	return 0;
}

LRESULT CTetris::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	DeleteObject(icon);
	return 0;
}

void CTetris::OnDraw(HDC hdc, int width, int height) {
	int wh = trayWindow.tidgetSize;
	StretchBitmap(hdc, 0, 0, wh, wh, icon);
}

void CTetris::OnClick() {
	//  츦 Ͽ . 
	CTetrisWin* pWin = new CTetrisWin();
	pWin->delegateDispose = true;
	CreateWindow(TEXT("CWindow"), L"Tetris", WS_OVERLAPPED | WS_SYSMENU | WS_BORDER | WS_CAPTION | WS_VISIBLE,
		10, 10, 100, 100,
		NULL, (HMENU)NULL, g_hInst, pWin);
}

// Ŭȿ ʱȭ 迭 ϴ  𸣰. C   . 
struct TetPoint {
	int x, y;
};
TetPoint Shape[][4][4] = {
	{ {0,0,1,0,2,0,-1,0}, {0,0,0,1,0,-1,0,-2}, {0,0,1,0,2,0,-1,0}, {0,0,0,1,0,-1,0,-2} },
	{ {0,0,1,0,0,1,1,1}, {0,0,1,0,0,1,1,1}, {0,0,1,0,0,1,1,1}, {0,0,1,0,0,1,1,1} },
	{ {0,0,-1,0,0,-1,1,-1}, {0,0,0,1,-1,0,-1,-1}, {0,0,-1,0,0,-1,1,-1}, {0,0,0,1,-1,0,-1,-1} },
	{ {0,0,-1,-1,0,-1,1,0}, {0,0,-1,0,-1,1,0,-1}, {0,0,-1,-1,0,-1,1,0}, {0,0,-1,0,-1,1,0,-1} },
	{ {0,0,-1,0,1,0,-1,-1}, {0,0,0,-1,0,1,-1,1}, {0,0,-1,0,1,0,1,1}, {0,0,0,-1,0,1,1,-1} },
	{ {0,0,1,0,-1,0,1,-1}, {0,0,0,1,0,-1,-1,-1}, {0,0,1,0,-1,0,-1,1}, {0,0,0,-1,0,1,1,1} },
};

LRESULT CTetrisWin::OnCreate(WPARAM wParam, LPARAM lParam)
{
	ts = 24;
	bw = 10;
	bh = 20;
	//memset(board, 0, sizeof(board));

	GameStatus = GAMEOVER;
	srand(GetTickCount());
	AdjustMainWindow();

	//    . 
	RECT wrt;
	GetWindowRect(mainWindow.win_hWnd, &wrt);
	MovePopupToSide(win_hWnd, wrt.left, wrt.top);

	// 0 ĭ, 1~6 . 7 
	COLORREF arTileColor[] = { 0, 0xff0000, 0x00ff00, 0x0000ff, 0xffff00, 0xff00ff, 0x00ffff, 0x000000 };
	for (int i = 1; i <= 7; i++) {
		arBrush[i] = CreateSolidBrush(arTileColor[i]);
	}

	return 0;
}

LRESULT CTetrisWin::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	for (int i = 1; i <= 7; i++) {
		DeleteObject(arBrush[i]);
	}
	return 0;
}

LRESULT CTetrisWin::OnTimer(WPARAM wParam, LPARAM lParam)
{
	if (MoveDown() == TRUE) {
		MakeNewBrick();
	}
	return 0;
}

void CTetrisWin::OnDraw(HDC hdc, int width, int height)
{
	int x, y, i;
	TCHAR str[128];

	FillRect(hdc, 0, 0, width, height, (HBRUSH)GetStockObject(LTGRAY_BRUSH));

	// ׵θ ׸
	for (x = 0; x < bw + 1; x++) {
		PrintTile(hdc, x, 0, WALL);
		PrintTile(hdc, x, bh + 1, WALL);
	}
	for (y = 0; y < bh + 2; y++) {
		PrintTile(hdc, 0, y, WALL);
		PrintTile(hdc, bw + 1, y, WALL);
	}

	//  ׸
	for (x = 1; x < bw + 1; x++) {
		for (y = 1; y < bh + 1; y++) {
			PrintTile(hdc, x, y, board[x][y]);
		}
	}

	// ̵  ׸
	if (GameStatus != GAMEOVER && brick != -1) {
		for (i = 0; i < 4; i++) {
			PrintTile(hdc, nx + Shape[brick][rot][i].x, ny + Shape[brick][rot][i].y, brick + 1);
		}
	}

	//   ׸
	for (x = bw + 3; x <= bw + 11; x++) {
		for (y = bh - 5; y <= bh + 1; y++) {
			if (x == bw + 3 || x == bw + 11 || y == bh - 5 || y == bh + 1) {
				PrintTile(hdc, x, y, WALL);
			} else {
				PrintTile(hdc, x, y, 0);
			}
		}
	}
	if (GameStatus != GAMEOVER) {
		for (i = 0; i < 4; i++) {
			PrintTile(hdc, bw + 7 + Shape[nbrick][0][i].x, bh - 2 + Shape[nbrick][0][i].y, nbrick + 1);
		}
	}

	//  
	HFONT oldFont;
	oldFont = (HFONT)SelectObject(hdc, mainFont[1]);
	SetBkMode(hdc, TRANSPARENT);
	lstrcpy(str, TEXT("1~9: ũ"));
	TextOutSz(hdc, (bw + 4) * ts, 30, str);
	lstrcpy(str, TEXT("Enter:/"));
	TextOutSz(hdc, (bw + 4) * ts, 60, str);
	wsprintf(str, TEXT(" : %d   "), score);
	TextOutSz(hdc, (bw + 4) * ts, 90, str);
	wsprintf(str, TEXT(" : %d    "), bricknum);
	TextOutSz(hdc, (bw + 4) * ts, 120, str);
	SelectObject(hdc, oldFont);
}

LRESULT CTetrisWin::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	int x, y;
	int trot;

	//   ο  ϴ Ű
	if (wParam == VK_RETURN) {
		if (GameStatus == GAMEOVER) {
			for (x = 0; x < bw + 2; x++) {
				for (y = 0; y < bh + 2; y++) {
					board[x][y] = (y == 0 || y == bh + 1 || x == 0 || x == bw + 1) ? WALL : EMPTY;
				}
			}
			score = 0;
			bricknum = 0;
			GameStatus = RUNNING;
			nbrick = random(sizeof(Shape) / sizeof(Shape[0]));
			MakeNewBrick();
			Interval = 1000;
			SetTimer(win_hWnd, 1, Interval, NULL);
		} else if (GameStatus == RUNNING) {
			GameStatus = PAUSE;
			KillTimer(win_hWnd, 1);
		} else if (GameStatus == PAUSE) {
			GameStatus = RUNNING;
			SetTimer(win_hWnd, 1, Interval, NULL);
		}
	}

	//  ũ   ٲ  ִ. 
	if (GameStatus == GAMEOVER && wParam >= '1' && wParam <= '9') {
		switch (wParam) {
		case '1':
			bw = 8; bh = 16; ts = 32;
			break;
		case '2':	// Ʈ
			bw = 10; bh = 20; ts = 24;
			break;
		case '3':
			bw = 12; bh = 24; ts = 40;
			break;
		case '4':
			bw = 15; bh = 30; ts = 28;
			break;
		case '5':	// FHD ִ ũ
			bw = 200; bh = 100; ts = 8;
			break;
		case '6':
			bw = 20; bh = 40; ts = 32;
			break;
		case '7':
			bw = 50; bh = 100; ts = 16;
			break;
		case '8':
			bw = 200; bh = 100; ts = 16;
			break;
		case '9':	// 4k ִ ũ
			bw = 400; bh = 200; ts = 8;
			break;
		}
		// ǿ ϴ   Ѵ. 
		memset(board, 0, sizeof(board));
		AdjustMainWindow();
		return 0;
	}

	//  Ű  ߿   . 
	if (GameStatus != RUNNING || brick == -1) {
		return 0;
	}

	switch (wParam) {
	case VK_LEFT:
		if (GetAround(nx - 1, ny, brick, rot) == EMPTY) {
			nx--;
			InvalidateRect(win_hWnd, NULL, FALSE);
		}
		break;
	case VK_RIGHT:
		if (GetAround(nx + 1, ny, brick, rot) == EMPTY) {
			nx++;
			InvalidateRect(win_hWnd, NULL, FALSE);
		}
		break;
	case VK_UP:
		trot = (rot == 3 ? 0 : rot + 1);
		if (GetAround(nx, ny, brick, trot) == EMPTY) {
			rot = trot;
			InvalidateRect(win_hWnd, NULL, FALSE);
		}
		break;
	case VK_DOWN:
		if (MoveDown() == TRUE) {
			MakeNewBrick();
		}
		break;
	case VK_SPACE:
		while (MoveDown() == FALSE) { ; }
		MakeNewBrick();
		break;
	}

	return 0;
}

void CTetrisWin::MakeNewBrick()
{
	bricknum++;
	brick = nbrick;
	nbrick = random(sizeof(Shape) / sizeof(Shape[0]));
	nx = bw / 2;
	ny = 3;
	rot = 0;
	InvalidateRect(win_hWnd, NULL, FALSE);
	if (GetAround(nx, ny, brick, rot) != EMPTY) {
		KillTimer(win_hWnd, 1);
		GameStatus = GAMEOVER;
		MessageBox(win_hWnd, L"GAME OVER", L"˸", MB_OK);
	}
}

int CTetrisWin::GetAround(int x, int y, int b, int r)
{
	int i, k = EMPTY;

	for (i = 0; i < 4; i++) {
		k = max(k, board[x + Shape[b][r][i].x][y + Shape[b][r][i].y]);
	}
	return k;
}
BOOL CTetrisWin::MoveDown()
{
	if (GetAround(nx, ny + 1, brick, rot) != EMPTY) {
		TestFull();
		return TRUE;
	}
	ny++;
	//  ׷     ش.
	InvalidateRect(win_hWnd, NULL, FALSE);
	//  ʹ  ִϸ̼ ʴ´. 
	if (bw < 100) {
		UpdateWindow(win_hWnd);
	}
	return FALSE;
}
void CTetrisWin::TestFull()
{
	int i, x, y, ty;
	int count = 0;
	static int arScoreInc[] = { 0,1,3,8,20 };

	for (i = 0; i < 4; i++) {
		board[nx + Shape[brick][rot][i].x][ny + Shape[brick][rot][i].y] = brick + 1;
	}
	// ̵    . 
	brick = -1;

	for (y = 1; y < bh + 1; y++) {
		for (x = 1; x < bw + 1; x++) {
			if (board[x][y] == EMPTY) break;
		}
		if (x == bw + 1) {
			count++;
			for (ty = y; ty > 1; ty--) {
				for (x = 1; x < bw + 1; x++) {
					board[x][ty] = board[x][ty - 1];
				}
			}
			InvalidateRect(win_hWnd, NULL, FALSE);
			UpdateWindow(win_hWnd);
			Sleep(150);
		}
	}
	score += arScoreInc[count];
	if (bricknum % 10 == 0 && Interval > 200) {
		Interval -= 50;
		SetTimer(win_hWnd, 1, Interval, NULL);
	}
}

void CTetrisWin::PrintTile(HDC hdc, int x, int y, int c)
{
	HPEN oldPen;
	HBRUSH oldBrush;
	RECT brt;

	if (c == EMPTY) {
		if (ts > 20) {
			Rectangle(hdc, x * ts + ts / 2 - 1, y * ts + ts / 2 - 1, x * ts + ts / 2 + 1, y * ts + ts / 2 + 1);
		} else {
			SetPixel(hdc, x * ts + ts / 2, y * ts + ts / 2, RGB(128, 128, 128));
		}
	} else {
		SetRect(&brt, x * ts, y * ts, x * ts + ts, y * ts + ts);

		oldBrush = (HBRUSH)SelectObject(hdc, arBrush[c]);
		oldPen = (HPEN)SelectObject(hdc, GetStockObject(NULL_PEN));
		Rectangle(hdc, brt.left, brt.top, brt.right, brt.bottom);
		SelectObject(hdc, oldBrush);
		SelectObject(hdc, oldPen);

		// ü ٷ ߴµ  ʰ  . 
		/*white = CreatePen(PS_SOLID, 1, RGB(200, 200, 200));
		gray = CreatePen(PS_SOLID, 1, RGB(32, 32, 32));
		oldPen = (HPEN)SelectObject(hdc, white);
		MoveToEx(hdc, brt.left, brt.top, NULL); LineTo(hdc, brt.left, brt.bottom);
		MoveToEx(hdc, brt.left, brt.top, NULL); LineTo(hdc, brt.right, brt.top);
		SelectObject(hdc, gray);
		MoveToEx(hdc, brt.left, brt.bottom, NULL); LineTo(hdc, brt.right, brt.bottom);
		MoveToEx(hdc, brt.right, brt.top, NULL); LineTo(hdc, brt.right, brt.bottom);
		SelectObject(hdc, oldPen);
		DeleteObject(white);
		DeleteObject(gray);
		*/
	}
}

void CTetrisWin::AdjustMainWindow()
{
	RECT crt;
	SetRect(&crt, 0, 0, (bw + 12) * ts, (bh + 2) * ts);
	AdjustWindowRect(&crt, WS_OVERLAPPED | WS_SYSMENU | WS_BORDER | WS_CAPTION, FALSE);
	SetWindowPos(win_hWnd, NULL, 0, 0, crt.right - crt.left, crt.bottom - crt.top,
		SWP_NOMOVE | SWP_NOZORDER);
}

LRESULT COneImage::OnCreate(WPARAM wParam, LPARAM lParam)
{
	nowImage = NULL;
	pViewer = NULL;

	setting.get(dget_id, L"imagePath", L"", imagePath, MAX_PATH);
	if (lstrlen(imagePath) != 0) {
		nowImage = Image::FromFile(imagePath);
		FixImageOrientation(nowImage);
	}
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_ONEIMAGE));

	return 0;
}

LRESULT COneImage::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	DeleteObject(icon);
	if (nowImage != NULL) {
		delete nowImage;
	}
	if (pViewer != NULL) {
		DestroyWindow(pViewer->win_hWnd);
	}

	return 0;
}

void COneImage::OnDraw(HDC hdc, int width, int height)
{
	int wh = trayWindow.tidgetSize;
	if (nowImage == NULL) {
		StretchBitmap(hdc, 0, 0, wh, wh, icon);
	} else {
		DrawImageCenter(hdc, nowImage, wh, wh, true, win_backBrush);
	}
}

void COneImage::OnClick()
{
	if (nowImage != NULL) {
		//  .
		if (pViewer == NULL) {
			pViewer = new CImageViewer();
			pViewer->delegateDispose = true;
			lstrcpy(pViewer->imagePath, imagePath);
			pViewer->Create(1, this);
		}
	}
}

// todo : ɼ ȭڸ      ư ġѴ.
// ̰ ϱ  Ŀ Ҿ ɼâ  ʵ å ؾ Ѵ. 
LRESULT COneImage::OnContextMenu(WPARAM wParam, LPARAM lParam)
{
	OPENFILENAME OFN;
	lstrcpy(imagePath, L"");
	memset(&OFN, 0, sizeof(OPENFILENAME));
	OFN.lStructSize = sizeof(OPENFILENAME);
	OFN.hwndOwner = win_hWnd;
	OFN.lpstrFilter = TEXT("̹ (jpg, png)\0*.jpg;*.png\0");
	OFN.lpstrFile = imagePath;
	OFN.nMaxFile = MAX_PATH;
	if (GetOpenFileName(&OFN) == 0) return 0;

	// ̹ Ǵ  Ѵ. 
	setting.set(dget_id, L"imagePath", imagePath);
	nowImage = Image::FromFile(imagePath);
	FixImageOrientation(nowImage);
	InvalidateRect(win_hWnd, NULL, TRUE);

	return 0;
}

//   ȣ   д. ׷    ִ. 
LRESULT COneImage::OnUser1(WPARAM wParam, LPARAM lParam)
{
	pViewer = NULL;
	return 0;
}

// ̹   todo - ⺻     Ŀ õõ  
// ŸƲ ϰ β  θ   巡ؼ ũ . ٴ 巡ؼ ̵
// 콺  ö󰡸 ϸ ݱ, ִȭ ư  . Ǵ   β ؼ ׻ ǥõ 
// Ȯ, , д, ȸ, ݻ,    . ̴  PgUp, PgDn ׺̼ ̵
void CImageViewer::Create(int parentType, CDget* parent)
{
	// ŸԿ  θ ĳ д.  ̴   迩 C ŸϷ  Ѵ.
	// CWindow    pParent ٷ  ȴ. 
	pParent = parent;
	this->parentType = parentType;
	switch (parentType) {
	case 0:
		pSlide = (CImageSlide*)pParent;
		break;
	case 1:
		pOneImage = (COneImage*)pParent;
		break;
	}

	CreateWindow(TEXT("CWindow"), L"̹ ", WS_POPUPWINDOW | WS_THICKFRAME | WS_CAPTION,
		0, 0, 0, 0,
		NULL, (HMENU)NULL, g_hInst, this);

	//   ġ о´. ϴ 븻 ǥ ʱ ǥ ´.
	prevWndPlace.length = sizeof(WINDOWPLACEMENT);
	prevWndPlace.flags = 0;
	prevWndPlace.rcNormalPosition.left = setting.get(pParent->dget_id, L"left", 10);
	prevWndPlace.rcNormalPosition.top = setting.get(pParent->dget_id, L"top", 10);
	prevWndPlace.rcNormalPosition.right = setting.get(pParent->dget_id, L"right", 650);
	prevWndPlace.rcNormalPosition.bottom = setting.get(pParent->dget_id, L"bottom", 490);
	prevWndPlace.showCmd = setting.get(pParent->dget_id, L"showCmd", SW_SHOWNORMAL);
	SetWindowPlacement(win_hWnd, &prevWndPlace);

	// ü ȭ ȯѴ. 
	fullScreen = setting.get(pParent->dget_id, L"fullScreen", false);
	if (fullScreen) {
		prevWndPlace.length = sizeof(WINDOWPLACEMENT);
		GetWindowPlacement(win_hWnd, &prevWndPlace);
		makeFullScreen(win_hWnd);
	}
}

LRESULT CImageViewer::OnLButtonDblClk(WPARAM wParam, LPARAM lParam)
{
	// ϴ Ŭ ִȭѴ.  ٸ  ü .
	toggleFullScreen();
	return 0;
}

LRESULT CImageViewer::OnUser1(WPARAM wParam, LPARAM lParam)
{
	// wParam   ̹ ǥѴ.
	if (nowImage != NULL) {
		delete nowImage;
	}
	nowIdx = (int)wParam;
	nowImage = Image::FromFile(pSlide->arImage[nowIdx].c_str());
	SetWindowText(win_hWnd, pSlide->arImage[nowIdx].c_str());
	InvalidateRect(win_hWnd, NULL, TRUE);
	return 0;
}

LRESULT CImageViewer::OnCreate(WPARAM wParam, LPARAM lParam)
{
	// ̴ idx ְ, OneImage θ  ش.
	LPCTSTR Path;
	if (parentType == 0) {
		Path = pSlide->arImage[nowIdx].c_str();
	} else {
	Path = imagePath;
	}
	nowImage = Image::FromFile(Path);
	FixImageOrientation(nowImage);
	SetWindowText(win_hWnd, Path);
	return 0;
}

LRESULT CImageViewer::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	if (nowImage != NULL) {
		delete nowImage;
	}

	//   ġ Ѵ. , ü ȭ ̸ ü ȭ  븻 ǥ Ѵ.
	if (fullScreen == false) {
		prevWndPlace.length = sizeof(WINDOWPLACEMENT);
		GetWindowPlacement(win_hWnd, &prevWndPlace);
	}

	setting.set(pParent->dget_id, L"left", prevWndPlace.rcNormalPosition.left);
	setting.set(pParent->dget_id, L"top", prevWndPlace.rcNormalPosition.top);
	setting.set(pParent->dget_id, L"right", prevWndPlace.rcNormalPosition.right);
	setting.set(pParent->dget_id, L"bottom", prevWndPlace.rcNormalPosition.bottom);
	setting.set(pParent->dget_id, L"showCmd", prevWndPlace.showCmd);
	setting.set(pParent->dget_id, L"fullScreen", fullScreen);

	// θ𿡰 ڽ  ˸. ׷ Ŭ ο    ִ. 
	SendMessage(pParent->win_hWnd, WM_USER + 1, 0, 0);

	// ڱ ڽ Ѵ. üʿ ȴ. -> ̷   NCDESTROY ó  ٿ. -> θ     ϴ. 
	// delete this;
	return 0;
}

LRESULT CImageViewer::OnPaint(WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	hdc = BeginPaint(win_hWnd, &ps);
	RECT crt;
	GetClientRect(win_hWnd, &crt);

	// θ  귯 
	DrawImageCenter(hdc, nowImage, crt.right, crt.bottom, true, pParent->win_backBrush);

	EndPaint(win_hWnd, &ps);
	return 0;
}

LRESULT CImageViewer::OnSize(WPARAM wParam, LPARAM lParam)
{
	InvalidateRect(win_hWnd, NULL, TRUE);
	return 0;
}

LRESULT CImageViewer::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	if (wParam == VK_ESCAPE) {
		DestroyWindow(win_hWnd);
	}
	if (wParam == VK_RETURN) {
		toggleFullScreen();
	}

	return 0;
}

void CImageViewer::toggleFullScreen()
{
	fullScreen = !fullScreen;
	if (fullScreen) {
		// ü ȭ ȯϱ  븻 ǥ ̸  д.
		prevWndPlace.length = sizeof(WINDOWPLACEMENT);
		GetWindowPlacement(win_hWnd, &prevWndPlace);
		makeFullScreen(win_hWnd);
	} else {
		SetWindowLongPtr(win_hWnd, GWL_STYLE, WS_POPUPWINDOW | WS_CAPTION | WS_THICKFRAME | WS_VISIBLE);
		SetWindowPos(win_hWnd, HWND_NOTOPMOST, 0, 0, 0, 0,
			SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED);
		// 븻 ǥ ٽ Ѵ. 
		prevWndPlace.length = sizeof(WINDOWPLACEMENT);
		SetWindowPlacement(win_hWnd, &prevWndPlace);
	}
}

LRESULT CEnumFont::OnCreate(WPARAM wParam, LPARAM lParam)
{
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_ENUMFONT));
	return 0;
}

LRESULT CEnumFont::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	DeleteObject(icon);
	return 0;
}

void CEnumFont::OnDraw(HDC hdc, int width, int height) {
	int wh = trayWindow.tidgetSize;
	StretchBitmap(hdc, 0, 0, wh, wh, icon);
}

void CEnumFont::OnClick() {
	CEnumFontWin* pWin = new CEnumFontWin();
	pWin->delegateDispose = true;

	CreateWindow(TEXT("CWindow"), L"Ʈ ",
		WS_OVERLAPPED | WS_SYSMENU | WS_BORDER | WS_CAPTION | WS_VISIBLE | WS_CLIPCHILDREN,
		100, 100, 900, 800,
		NULL, (HMENU)NULL, g_hInst, pWin);
}

LRESULT CEnumFontWin::OnMessage(UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	HFONT hFont, oldFont;
	LPMEASUREITEMSTRUCT lpmis;
	LPDRAWITEMSTRUCT lpdis;
	HBRUSH bkBrush;
	LOGFONT tmpFont;

	switch (iMessage)
	{
	case WM_MEASUREITEM:
		lpmis = (LPMEASUREITEMSTRUCT)lParam;
		lpmis->itemHeight = 30;
		return 0;
	case WM_DRAWITEM:
		lpdis = (LPDRAWITEMSTRUCT)lParam;

		// õ ׸ Ķ 濡  Ѵ.
		if (lpdis->itemState & ODS_SELECTED) {
			bkBrush = GetSysColorBrush(COLOR_HIGHLIGHT);
			SetBkColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHT));
			SetTextColor(lpdis->hDC, GetSysColor(COLOR_WINDOW));
		} else {
			bkBrush = GetSysColorBrush(COLOR_WINDOW);
			SetBkColor(lpdis->hDC, GetSysColor(COLOR_WINDOW));
			SetTextColor(lpdis->hDC, GetSysColor(COLOR_WINDOWTEXT));
		}
		FillRect(lpdis->hDC, &lpdis->rcItem, bkBrush);

		// 25 ȼ ̷ ۲ ̸ Ѵ.
		tmpFont = arFont[lpdis->itemID];
		tmpFont.lfHeight = 25;
		tmpFont.lfWidth = 0;
		hFont = CreateFontIndirect(&tmpFont);
		oldFont = (HFONT)SelectObject(lpdis->hDC, hFont);

		TextOutSz(lpdis->hDC, lpdis->rcItem.left + 5, lpdis->rcItem.top + 2,
			arFont[lpdis->itemID].lfFaceName);
		SelectObject(lpdis->hDC, oldFont);
		DeleteObject(hFont);
		return 0;
	}

	return CWindow::OnMessage(iMessage, wParam, lParam);
}

LRESULT CEnumFontWin::OnCreate(WPARAM wParam, LPARAM lParam)
{
	int left, top;
	left = setting.get(L"EnumFont", L"left", 100);
	top = setting.get(L"EnumFont", L"top", 100);
	SetWindowPos(win_hWnd, NULL, left, top, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
	EnsureVisible(win_hWnd);

	SetBackColor(GetSysColor(COLOR_BTNFACE));
	hList = CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("listbox"), NULL,
		WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_NOTIFY | LBS_OWNERDRAWFIXED,
		10, 10, 280, 550, win_hWnd, (HMENU)EF_LIST, g_hInst, NULL);
	hChkAt = CreateWindow(TEXT("button"), TEXT("@ Ʈ "), WS_CHILD | WS_VISIBLE |
		BS_AUTOCHECKBOX, 10, 570, 120, 25, win_hWnd, (HMENU)EF_AT, g_hInst, NULL);
	hChkSort = CreateWindow(TEXT("button"), TEXT("Ʈ "), WS_CHILD | WS_VISIBLE |
		BS_AUTOCHECKBOX, 10, 600, 120, 25, win_hWnd, (HMENU)EF_SORT, g_hInst, NULL);
	hStNum = CreateWindow(TEXT("static"), TEXT("߰ߵ Ʈ"), WS_CHILD | WS_VISIBLE,
		10, 630, 120, 25, win_hWnd, (HMENU)EF_NUM, g_hInst, NULL);
	hBtnApi = CreateWindow(TEXT("button"), TEXT("API ڵ "), WS_CHILD | WS_VISIBLE |
		BS_PUSHBUTTON, 300, 450, 120, 30, win_hWnd, (HMENU)EF_API, g_hInst, NULL);
	hEdApi = CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("edit"), NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_AUTOHSCROLL | ES_MULTILINE,
		300, 490, 580, 60, win_hWnd, (HMENU)EF_EDAPI, g_hInst, NULL);
	SendMessage(hEdApi, WM_SETFONT, (WPARAM)mainFont[1], MAKELPARAM(FALSE, 0));
	hBtnMfc = CreateWindow(TEXT("button"), TEXT("MFC ڵ "), WS_CHILD | WS_VISIBLE |
		BS_PUSHBUTTON, 300, 560, 120, 30, win_hWnd, (HMENU)EF_MFC, g_hInst, NULL);
	hEdMfc = CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("edit"), NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_AUTOHSCROLL | ES_MULTILINE,
		300, 590, 580, 60, win_hWnd, (HMENU)EF_EDMFC, g_hInst, NULL);
	SendMessage(hEdMfc, WM_SETFONT, (WPARAM)mainFont[1], MAKELPARAM(FALSE, 0));

	SendMessage(hChkSort, BM_SETCHECK, BST_CHECKED, 0);
	EnumFont();

	return 0;
}

LRESULT CEnumFontWin::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	RECT wrt;
	GetWindowRect(win_hWnd, &wrt);
	setting.set(L"EnumFont", L"left", wrt.left);
	setting.set(L"EnumFont", L"top", wrt.top);

	return 0;
}

void CEnumFontWin::OnDraw(HDC hdc, int width, int height)
{
	int TabPos[] = { 250 };
	TCHAR str[256], str2[128];
	HFONT hFont, oldFont;

	SetBkMode(hdc, TRANSPARENT);

	wsprintf(str, L"Height\t: %d", arFont[nowFont].lfHeight);
	TabbedTextOut(hdc, 300, 20, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Width\t: %d", arFont[nowFont].lfWidth);
	TabbedTextOut(hdc, 300, 40, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Escapement\t: %d", arFont[nowFont].lfEscapement);
	TabbedTextOut(hdc, 300, 60, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Orientation\t: %d", arFont[nowFont].lfOrientation);
	TabbedTextOut(hdc, 300, 80, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Weight\t: %d", arFont[nowFont].lfWeight);
	TabbedTextOut(hdc, 300, 100, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Italic\t: %s", arFont[nowFont].lfItalic ? L"TRUE" : L"FALSE");
	TabbedTextOut(hdc, 300, 120, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Underline\t: %s", arFont[nowFont].lfUnderline ? L"TRUE" : L"FALSE");
	TabbedTextOut(hdc, 300, 140, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"StrikeOut\t: %s", arFont[nowFont].lfStrikeOut ? L"TRUE" : L"FALSE");
	TabbedTextOut(hdc, 300, 160, str, lstrlen(str), 1, TabPos, 0);

	GetCharSetStr(arFont[nowFont].lfCharSet, str2);
	wsprintf(str, L"CharSet\t: %s", str2);
	TabbedTextOut(hdc, 300, 180, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"OutPrecision\t: %d", arFont[nowFont].lfOutPrecision);
	TabbedTextOut(hdc, 300, 200, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"ClipPrecision\t: %d", arFont[nowFont].lfClipPrecision);
	TabbedTextOut(hdc, 300, 220, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"Quality\t: %d", arFont[nowFont].lfQuality);
	TabbedTextOut(hdc, 300, 240, str, lstrlen(str), 1, TabPos, 0);

	GetPitchFamStr(arFont[nowFont].lfPitchAndFamily, str2);
	wsprintf(str, L"PitchAndFamily\t: %s", str2);
	TabbedTextOut(hdc, 300, 260, str, lstrlen(str), 1, TabPos, 0);

	wsprintf(str, L"FaceName\t: %s", arFont[nowFont].lfFaceName);
	TabbedTextOut(hdc, 300, 280, str, lstrlen(str), 1, TabPos, 0);

	// Ʈ Ÿ Ѵ.   LOGFONT ü .
	switch (arFontType[nowFont]) {
	case 1:
		wsprintf(str, L"FontType\t: RASTER_FONTTYPE");
		break;
	case 2:
		wsprintf(str, L"FontType\t: DEVICE_FONTTYPE");
		break;
	case 4:
		wsprintf(str, L"FontType\t: TRUETYPE_FONTTYPE");
		break;
	default:
		wsprintf(str, L"FontType\t: UnKnown(%d)", arFontType[nowFont]);
	}
	TabbedTextOut(hdc, 300, 300, str, lstrlen(str), 1, TabPos, 0);

	// Ʈ  Ѵ.
	lstrcpy(str, L"English ѱ 1234!@#$");
	hFont = CreateFontIndirect(&arFont[nowFont]);
	oldFont = (HFONT)SelectObject(hdc, hFont);
	TextOutSz(hdc, 300, 340, str);
	SelectObject(hdc, oldFont);
	DeleteObject(hFont);
}

LRESULT CEnumFontWin::OnCommand(WPARAM wParam, LPARAM lParam)
{
	TCHAR content[1024];

	switch (LOWORD(wParam)) {
	case EF_AT:
	case EF_SORT:
		EnumFont();
		break;
	case EF_LIST:
		switch (HIWORD(wParam)) {
		case LBN_SELCHANGE:
			nowFont = SendMessage(hList, LB_GETCURSEL, 0, 0);
			SetCreateFontCommand();
			InvalidateRect(win_hWnd, NULL, TRUE);
			break;
		}
		break;
	case EF_API:
		GetWindowText(hEdApi, content, 1024);
		CopyClipboardText(content);
		break;
	case EF_MFC:
		GetWindowText(hEdMfc, content, 1024);
		CopyClipboardText(content);
		break;
	}

	return 0;
}

void CEnumFontWin::EnumFont()
{
	HDC hdc;
	int i;

	numFont = 0;
	nowFont = -1;
	includeAt = (SendMessage(hChkAt, BM_GETCHECK, 0, 0) == BST_CHECKED);
	hdc = GetDC(mainWindow.win_hWnd);
	EnumFontFamilies(hdc, NULL, (FONTENUMPROC)EnumFamCallBack, (LPARAM)this);
	ReleaseDC(mainWindow.win_hWnd, hdc);

	if (SendMessage(hChkSort, BM_GETCHECK, 0, 0) == BST_CHECKED) {
		qsort(&arFont[0], numFont, sizeof(LOGFONT), CEnumFontWin::compareFace);
	}

	// ߰ߵ  Ʈ Ÿ ̽ Ʈ ڽ Ѵ.
	SendMessage(hList, LB_RESETCONTENT, 0, 0);
	for (i = 0; i < numFont; i++) {
		SendMessage(hList, LB_ADDSTRING, 0, (LPARAM)arFont[i].lfFaceName);
	}

	TCHAR szNum[111];
	wsprintf(szNum, L"߰ߵ Ʈ : %d", numFont);
	SetWindowText(hStNum, szNum);

	nowFont = 0;
	SendMessage(hList, LB_SETCURSEL, nowFont, 0);
	SendMessage(win_hWnd, WM_COMMAND, MAKEWPARAM(EF_LIST, LBN_SELCHANGE), (LPARAM)hList);
	InvalidateRect(win_hWnd, NULL, TRUE);
}

int CALLBACK CEnumFontWin::EnumFamCallBack(ENUMLOGFONT FAR* lpelf,
	NEWTEXTMETRIC FAR* lpntm, int FontType, LPARAM lParam)
{
	CEnumFontWin* pWin = (CEnumFontWin*)lParam;

	if (pWin->includeAt == false && _tcsrchr(lpelf->elfLogFont.lfFaceName, '@') != nullptr) {
		return TRUE;
	}

	if (pWin->numFont < CEnumFontWin::MAXFONT) {
		pWin->arFont[pWin->numFont] = lpelf->elfLogFont;
		pWin->arFontType[pWin->numFont] = FontType;
		pWin->numFont++;
		return TRUE;
	} else {
		return FALSE;
	}
}

int CEnumFontWin::compareFace(const void* a, const void* b)
{
	LOGFONT* aFont = (LOGFONT*)a;
	LOGFONT* bFont = (LOGFONT*)b;

	return lstrcmp(aFont->lfFaceName, bFont->lfFaceName);
}

void CEnumFontWin::GetCharSetStr(int CharSet, TCHAR* str)
{
	switch (CharSet) {
	case 0:lstrcpy(str, L"ANSI_CHARSET"); break;
	case 1:lstrcpy(str, L"DEFAULT_CHARSET"); break;
	case 2:lstrcpy(str, L"SYMBOL_CHARSET"); break;
	case 128:lstrcpy(str, L"SHIFTJIS_CHARSET"); break;
	case 129:lstrcpy(str, L"HANGUL_CHARSET"); break;
	case 136:lstrcpy(str, L"CHINESEBIG5_CHARSET"); break;
	case 255:lstrcpy(str, L"OEM_CHARSET"); break;
	case 130:lstrcpy(str, L"JOHAB_CHARSET"); break;
	case 177:lstrcpy(str, L"HEBREW_CHARSET"); break;
	case 178:lstrcpy(str, L"ARABIC_CHARSET"); break;
	case 161:lstrcpy(str, L"GREEK_CHARSET"); break;
	case 162:lstrcpy(str, L"TURKISH_CHARSET"); break;
	case 163:lstrcpy(str, L"VIETNAMESE_CHARSET"); break;
	case 222:lstrcpy(str, L"THAI_CHARSET"); break;
	case 238:lstrcpy(str, L"EASTEUROPE_CHARSET"); break;
	case 204:lstrcpy(str, L"RUSSIAN_CHARSET"); break;
	case 77:lstrcpy(str, L"MAC_CHARSET"); break;
	case 186:lstrcpy(str, L"BALTIC_CHARSET"); break;
	default:lstrcpy(str, L"Etc"); break;
	}
}

void CEnumFontWin::GetPitchFamStr(int Pitch, TCHAR* str)
{
	switch (Pitch & 0x0f) {
	case 0:lstrcpy(str, L"DEFAULT_PITCH");
		break;
	case 1:lstrcpy(str, L"FIXED_PITCH");
		break;
	case 2:lstrcpy(str, L"VARIABLE_PITCH");
		break;
	default:lstrcpy(str, L"Etc");
	}
	switch (Pitch & 0xf0) {
	case 0x00:lstrcat(str, L" | FF_DONTCARE");
		break;
	case 0x10:lstrcat(str, L" | FF_ROMAN");
		break;
	case 0x20:lstrcat(str, L" | FF_SWISS");
		break;
	case 0x30:lstrcat(str, L" | FF_MODERN");
		break;
	case 0x40:lstrcat(str, L" | FF_SCRIPT");
		break;
	case 0x50:lstrcat(str, L" | FF_DECORATIVE");
		break;
	}
}

void CEnumFontWin::SetCreateFontCommand()
{
	TCHAR Argu[128];
	TCHAR Api[1024];
	TCHAR Mfc[1024];
	TCHAR CharSetStr[32];
	TCHAR PitchFamStr[32];

	GetCharSetStr(arFont[nowFont].lfCharSet, CharSetStr);
	GetPitchFamStr(arFont[nowFont].lfPitchAndFamily, PitchFamStr);
	wsprintf(Argu, L"(%d,0,0,0,0,0,0,0,%s,3,2,1,\r\n\t%s,TEXT(\"%s\"));",
		arFont[nowFont].lfHeight, CharSetStr, PitchFamStr, arFont[nowFont].lfFaceName);

	wsprintf(Api, L"hFont = CreateFont(%d,0,0,0,0,0,0,0,%s,3,2,1,"
		"\r\n\t%s,TEXT(\"%s\"));",
		arFont[nowFont].lfHeight, CharSetStr, PitchFamStr, arFont[nowFont].lfFaceName);
	wsprintf(Api, L"hFont=CreateFont%s", Argu);
	SetWindowText(hEdApi, Api);
	wsprintf(Mfc, L"Font.CreateFont%s\r\n", Argu);
	SetWindowText(hEdMfc, Mfc);
}

LRESULT CSpy::OnCreate(WPARAM wParam, LPARAM lParam)
{
	icon = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_SPY));
	return 0;
}

LRESULT CSpy::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	DeleteObject(icon);
	return 0;
}

void CSpy::OnDraw(HDC hdc, int width, int height) {
	int wh = trayWindow.tidgetSize;
	StretchBitmap(hdc, 0, 0, wh, wh, icon);
}

void CSpy::OnClick() {
	CSpyWin* pWin = new CSpyWin();
	pWin->delegateDispose = true;

	CreateWindow(TEXT("CWindow"), L"Spy",
		WS_OVERLAPPED | WS_SYSMENU | WS_BORDER | WS_CAPTION | WS_VISIBLE | WS_CLIPCHILDREN,
		100, 100, 635, 650,
		NULL, (HMENU)NULL, g_hInst, pWin);
}

LRESULT CSpyWin::OnMessage(UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch (iMessage)
	{
	case 0:
		return 0;
	}

	return CWindow::OnMessage(iMessage, wParam, lParam);
}

LRESULT CSpyWin::OnCreate(WPARAM wParam, LPARAM lParam)
{
	int left, top;
	left = setting.get(L"Spy", L"left", 100);
	top = setting.get(L"Spy", L"top", 100);
	SetWindowPos(win_hWnd, NULL, left, top, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
	EnsureVisible(win_hWnd);

	SetBackColor(GetSysColor(COLOR_BTNFACE));
	CreateWindow(TEXT("static"), TEXT(" "), WS_CHILD | WS_VISIBLE,
		10, 10, 300, 25, win_hWnd, (HMENU)-1, g_hInst, NULL);
	hChkPause = CreateWindow(TEXT("button"), TEXT("Ͻ "), WS_CHILD | WS_VISIBLE |
		BS_AUTOCHECKBOX, 510, 10, 100, 25, win_hWnd, (HMENU)SPY_PAUSE, g_hInst, NULL);
	hEditInfo = CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("edit"), NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_MULTILINE | WS_VSCROLL,
		10, 40, 600, 250, win_hWnd, (HMENU)SPY_INFO, g_hInst, NULL);
	SendMessage(hEditInfo, WM_SETFONT, (WPARAM)mainFont[1], MAKELPARAM(FALSE, 0));
	CreateWindow(TEXT("static"), TEXT(" "), WS_CHILD | WS_VISIBLE,
		10, 310, 300, 25, win_hWnd, (HMENU)-1, g_hInst, NULL);
	hEditHier = CreateWindowEx(WS_EX_CLIENTEDGE, TEXT("edit"), NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_MULTILINE | WS_VSCROLL,
		10, 340, 600, 250, win_hWnd, (HMENU)SPY_HIER, g_hInst, NULL);
	SendMessage(hEditHier, WM_SETFONT, (WPARAM)mainFont[1], MAKELPARAM(FALSE, 0));

	// 1ʿ ѹ ϵ  ڰ   ֵ ɼȭ
	TimerInterval = 1000;
	SetTimer(win_hWnd, 1, TimerInterval, NULL);
	SendMessage(win_hWnd, WM_TIMER, 1, 0);
	return 0;
}

LRESULT CSpyWin::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	RECT wrt;
	GetWindowRect(win_hWnd, &wrt);
	setting.set(L"Spy", L"left", wrt.left);
	setting.set(L"Spy", L"top", wrt.top);

	KillTimer(win_hWnd, 1);
	return 0;
}

LRESULT CSpyWin::OnTimer(WPARAM wParam, LPARAM lParam)
{
	POINT pt;
	RECT wrt, crt;
	HWND hWndPoint;
	HWND hWndParent;
	DWORD dwStyle, dwExStyle;
	TCHAR sCaption[256], sClass[256];
	TCHAR sStyle[256], sExStyle[256];
	TCHAR sLine[256];
	TCHAR sText[1024];

	GetCursorPos(&pt);
	GetWindowRect(win_hWnd, &wrt);
	hWndPoint = WindowFromPoint(pt);
	bool pause = (SendMessage(hChkPause, BM_GETCHECK, 0, 0) == BST_CHECKED);

	//   ̰ų   ų Ͻ  ̸   ʴ´.
	if (PtInRect(&wrt, pt) || hWndPoint == hWndOld || pause) {
		return 0;
	}
	hWndOld = hWndPoint;

	if (hWndPoint == NULL) {
		lstrcpy(sText, TEXT("찡 . ũž Ű "));
	} else {
		GetWindowText(hWndPoint, sCaption, 256);
		GetClassName(hWndPoint, sClass, 256);
		hWndParent = GetParent(hWndPoint);
		GetWindowRect(hWndPoint, &crt);
		dwStyle = GetWindowLong(hWndPoint, GWL_STYLE);
		GetStyleString(dwStyle, sStyle);
		dwExStyle = GetWindowLong(hWndPoint, GWL_EXSTYLE);
		GetExStyleString(dwExStyle, sExStyle);

		wsprintf(sText,
			TEXT("ڵ : %d(0x%x)\r\n")
			TEXT("Ŭ : %s\r\n")
			TEXT("ĸ : %s\r\n")
			TEXT("θ : %d(0x%x)\r\n")
			TEXT(" : (%d,%d) - (%d,%d), %d*%d\r\n")
			TEXT("Ÿ : 0X%X\r\n    %s\r\n")
			TEXT("Ȯ Ÿ : 0X%X\r\n    %s\r\n"),
			(int)hWndPoint, (int)hWndPoint, sClass, sCaption, (int)hWndParent, (int)hWndParent,
			crt.left, crt.top, crt.right, crt.bottom,
			crt.right - crt.left, crt.bottom - crt.top,
			dwStyle, sStyle, dwExStyle, sExStyle);
	}
	SetWindowText(hEditInfo, sText);

	// todo :   ׷  ְ   ٸ     ۼ
	wstring ancestor = L"";
	HWND hwnd = hWndPoint;
	for (;;) {
		hwnd = GetParent(hwnd);
		if (hwnd == NULL) {
			HWND hdesk = GetDesktopWindow();
			lstrcpy(sCaption, L"Desktop");
			GetWindowRect(hdesk, &crt);
		} else {
			GetWindowText(hwnd, sCaption, 256);
			GetWindowRect(hwnd, &crt);
		}
		wsprintf(sLine, L"%d(%s) (%d,%d) - (%d,%d), %d*%d\r\n",
			(int)hwnd, sCaption,
			crt.left, crt.top, crt.right, crt.bottom,
			crt.right - crt.left, crt.bottom - crt.top
		);
		ancestor = sLine + ancestor;
		if (hwnd == NULL) break;
	}

	wstring childs = L"";
	EnumChildWindows(hWndPoint, EnumChildProc, (LPARAM)&childs);

	ancestor = L" \r\n" + ancestor + L"\r\nڽ \r\n" + childs;
	SetWindowText(hEditHier, ancestor.c_str());

	return 0;
}

#define TEST(n) (dwStyle & (1 << n))
void CSpyWin::GetStyleString(DWORD dwStyle, TCHAR* sStyle)
{
	if (TEST(31)) lstrcpy(sStyle, TEXT("POPUP"));
	else if (TEST(30)) lstrcpy(sStyle, TEXT("CHILD"));
	else lstrcpy(sStyle, TEXT("OVERLAPPED"));

	if (TEST(29)) lstrcat(sStyle, TEXT(" | MINIMIZE"));
	if (TEST(28)) lstrcat(sStyle, TEXT(" | VISIBLE"));
	if (TEST(27)) lstrcat(sStyle, TEXT(" | DISABLED"));
	if (TEST(26)) lstrcat(sStyle, TEXT(" | CLIPSIBLINGS"));
	if (TEST(25)) lstrcat(sStyle, TEXT(" | CLIPCHILDREN"));
	if (TEST(24)) lstrcat(sStyle, TEXT(" | MAXIMIZE"));
	if (TEST(23)) lstrcat(sStyle, TEXT(" | BORDER"));
	if (TEST(22)) lstrcat(sStyle, TEXT(" | DLGFRAME"));
	// lstrcat(sStyle, L"\r\n    ");
	if (TEST(21)) lstrcat(sStyle, TEXT(" | VSCROLL"));
	if (TEST(20)) lstrcat(sStyle, TEXT(" | HSCROLL"));
	if (TEST(19)) lstrcat(sStyle, TEXT(" | SYSMENU"));
	if (TEST(18)) lstrcat(sStyle, TEXT(" | THICKFRAME"));
	if (TEST(17)) lstrcat(sStyle, TEXT(" | MINIMIZEBOX"));
	if (TEST(16)) lstrcat(sStyle, TEXT(" | MAXIMIZEBOX"));
}

void CSpyWin::GetExStyleString(DWORD dwExStyle, TCHAR* sExStyle)
{
	if (dwExStyle & 0x00001000L) lstrcpy(sExStyle, TEXT("RIGHT"));
	else lstrcpy(sExStyle, TEXT("LEFT"));

	if (dwExStyle & 0x00000010L) lstrcat(sExStyle, TEXT(" | ACCEPTFILES"));
	if (dwExStyle & 0x00040000L) lstrcat(sExStyle, TEXT(" | APPWINDOW"));
	if (dwExStyle & 0x00000200L) lstrcat(sExStyle, TEXT(" | CLIENTEDGE"));
	if (dwExStyle & 0x02000000L) lstrcat(sExStyle, TEXT(" | COMPOSITED"));
	if (dwExStyle & 0x00000400L) lstrcat(sExStyle, TEXT(" | CONTEXTHELP"));
	if (dwExStyle & 0x00010000L) lstrcat(sExStyle, TEXT(" | CONTROLPARENT"));
	if (dwExStyle & 0x00000001L) lstrcat(sExStyle, TEXT(" | DLGMODALFRAME"));
	if (dwExStyle & 0x00080000L) lstrcat(sExStyle, TEXT(" | LAYERED"));
	if (dwExStyle & 0x00400000L) lstrcat(sExStyle, TEXT(" | LAYOUTRTL"));
	if (dwExStyle & 0x00004000L) lstrcat(sExStyle, TEXT(" | LEFTSCROLLBAR"));
	if (dwExStyle & 0x00000040L) lstrcat(sExStyle, TEXT(" | MDICHILD"));
	if (dwExStyle & 0x08000000L) lstrcat(sExStyle, TEXT(" | NOACTIVATE"));
	if (dwExStyle & 0x00100000L) lstrcat(sExStyle, TEXT(" | NOINHERITLAYOUT"));
	if (dwExStyle & 0x00000004L) lstrcat(sExStyle, TEXT(" | NOPARENTNOTIFY"));
	if (dwExStyle & 0x00200000L) lstrcat(sExStyle, TEXT(" | NOREDIRECTIONBITMAP"));
	if (dwExStyle & 0x00002000L) lstrcat(sExStyle, TEXT(" | RTLREADING"));
	if (dwExStyle & 0x00020000L) lstrcat(sExStyle, TEXT(" | STATICEDGE"));
	if (dwExStyle & 0x00000080L) lstrcat(sExStyle, TEXT(" | TOOLWINDOW"));
	if (dwExStyle & 0x00000008L) lstrcat(sExStyle, TEXT(" | TOPMOST"));
	if (dwExStyle & 0x00000020L) lstrcat(sExStyle, TEXT(" | TRANSPARENT"));
	if (dwExStyle & 0x00000100L) lstrcat(sExStyle, TEXT(" | WINDOWEDGE"));
}

BOOL CALLBACK CSpyWin::EnumChildProc(_In_ HWND hwnd, _In_ LPARAM lParam)
{
	TCHAR sCaption[256];
	TCHAR sLine[256];
	RECT crt;
	wstring* pChilds = (wstring*)lParam;

	GetWindowText(hwnd, sCaption, 256);
	GetWindowRect(hwnd, &crt);
	wsprintf(sLine, L"%d(%s) (%d,%d) - (%d,%d), %d*%d\r\n",
		(int)hwnd, sCaption,
		crt.left, crt.top, crt.right, crt.bottom,
		crt.right - crt.left, crt.bottom - crt.top
	);
	(*pChilds) += sLine;

	return TRUE;
}

