#include "pch.h"

// ü    
map<HWND, CWindow*> * CWindow::pWinMap;

CWindow::CWindow() {
	SetBackColor(RGB(255, 255, 255));
	win_dblBuffer = nullptr;

	if (pWinMap == nullptr) {
		pWinMap = new map<HWND, CWindow*>();
		WNDCLASS WndClass;

		WndClass.cbClsExtra = 0;
		WndClass.cbWndExtra = 0;
		WndClass.hbrBackground = NULL;
		WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
		WndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
		WndClass.hInstance = GetModuleHandle(NULL);
		WndClass.lpfnWndProc = (WNDPROC)CWindow::CWindowProc;
		WndClass.lpszClassName = L"CWindow";
		WndClass.lpszMenuName = NULL;
		WndClass.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
		RegisterClass(&WndClass);
	}
}

void CWindow::Dispose() {
	if (win_backBrush != nullptr) DeleteObject(win_backBrush);
	if (win_dblBuffer != nullptr) DeleteObject(win_dblBuffer);

	if (pWinMap != nullptr) {
		pWinMap->erase(win_hWnd);
	}
}

void CWindow::Create(HWND hParent, DWORD style, int child_id, LPCTSTR title) {
	CreateWindow(TEXT("CWindow"), title, style,
		0, 0, 0, 0, hParent, (HMENU)(ULONG_PTR)child_id, g_hInst, this);
}

void CWindow::SetBackColor(COLORREF color) {
	win_backColor = color;
	if (win_backBrush != nullptr) DeleteObject(win_backBrush);
	win_backBrush = CreateSolidBrush(color);
}

void CWindow::DeleteMap() {
	delete pWinMap;
	pWinMap = nullptr;
}

LRESULT CWindow::OnMessage(UINT iMessage, WPARAM wParam, LPARAM lParam) {
	switch (iMessage)
	{
	case WM_CREATE:
		return OnCreate(wParam, lParam);
	case WM_DESTROY:
		return OnDestroy(wParam, lParam);
	case WM_SIZE:
		return OnSize(wParam, lParam);
	case WM_PAINT:
		return OnPaint(wParam, lParam);
	case WM_TIMER:
		return OnTimer(wParam, lParam);
	case WM_COMMAND:
		return OnCommand(wParam, lParam);
	case WM_CONTEXTMENU:
		return OnContextMenu(wParam, lParam);
	case WM_KEYDOWN:
		return OnKeyDown(wParam, lParam);
	case WM_LBUTTONDOWN:
		return OnLButtonDown(wParam, lParam);
	case WM_MOUSEMOVE:
		return OnMouseMove(wParam, lParam);
	case WM_LBUTTONUP:
		return OnLButtonUp(wParam, lParam);
	case WM_LBUTTONDBLCLK:
		return OnLButtonDblClk(wParam, lParam);
	case WM_MOUSEWHEEL:
		return OnMouseWheel(wParam, lParam);
	case WM_USER + 1:
		return OnUser1(wParam, lParam);
	case WM_USER + 2:
		return OnUser2(wParam, lParam);
	case WM_DROPFILES:
		return OnDropFiles(wParam, lParam);
	}

	LRESULT ret = DefWindowProc(win_hWnd, iMessage, wParam, lParam);
	if (delegateDispose && iMessage == WM_NCDESTROY) {
		delete this;
	}
	return ret;
}

LRESULT CWindow::OnPaint(WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	HDC hMemDC;
	HBITMAP oldBit;

	RECT crt;
	GetClientRect(win_hWnd, &crt);
	hdc = BeginPaint(win_hWnd, &ps);
	if (win_dblBuffer == nullptr) {
		win_dblBuffer = CreateCompatibleBitmap(hdc, crt.right, crt.bottom);
	}
	hMemDC = CreateCompatibleDC(hdc);
	oldBit = (HBITMAP)SelectObject(hMemDC, win_dblBuffer);
	FillRect(hMemDC, &crt, win_backBrush);

	OnDraw(hMemDC, crt.right, crt.bottom);

	SelectObject(hMemDC, oldBit);
	DeleteDC(hMemDC);
	DrawBitmap(hdc, 0, 0, win_dblBuffer);
	EndPaint(win_hWnd, &ps);
	return 0;
}

LRESULT CWindow::OnSize(WPARAM wParam, LPARAM lParam)
{
	if (win_dblBuffer != nullptr) {
		DeleteObject(win_dblBuffer);
		win_dblBuffer = nullptr;
	}
	return 0;
}

LRESULT CALLBACK CWindow::CWindowProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	CWindow* pWin = nullptr;

	if (pWinMap->find(hWnd) != pWinMap->end()) {
		pWin = (*pWinMap)[hWnd];
	}

	if (pWin == nullptr) {
		if (iMessage == WM_GETMINMAXINFO) {
			return DefWindowProc(hWnd, iMessage, wParam, lParam);
		}
		pWin = (CWindow*)((LPCREATESTRUCT)lParam)->lpCreateParams;

		(*pWinMap)[hWnd] = pWin;
		pWin->win_hWnd = hWnd;
		Trace(L"%d  .  ũ = %d", hWnd, pWinMap->size());
	}

	return pWin->OnMessage(iMessage, wParam, lParam);
}





